@@ -15,6 +15,14 @@
 ** correctly populates and syncs a journal file before writing to a
 ** corresponding database file.
 **
+** $Id: test_journal.c,v 1.11 2009/02/12 09:11:56 danielk1977 Exp $
+*/
+#if SQLITE_TEST          /* This file is used for testing only */
+
+#include "sqlite3.h"
+#include "sqliteInt.h"
+
+/*
 ** INTERFACE
 **
 **   The public interface to this wrapper VFS is two functions:
@@ -42,7 +50,7 @@
 **     c) The set of page numbers corresponding to free-list leaf pages.
 **     d) A check-sum for every page in the database file.
 **
-**   The start of a write-transaction is deemed to have occurred when a 
+**   The start of a write-transaction is deemed to have occured when a 
 **   28-byte journal header is written to byte offset 0 of the journal 
 **   file.
 **
@@ -93,10 +101,6 @@
 **
 **     c) The journal file is deleted using xDelete.
 */
-#if SQLITE_TEST          /* This file is used for testing only */
-
-#include "sqlite3.h"
-#include "sqliteInt.h"
 
 /*
 ** Maximum pathname length supported by the jt backend.
@@ -159,10 +163,9 @@ static void jtDlClose(sqlite3_vfs*, void*);
 static int jtRandomness(sqlite3_vfs*, int nByte, char *zOut);
 static int jtSleep(sqlite3_vfs*, int microseconds);
 static int jtCurrentTime(sqlite3_vfs*, double*);
-static int jtCurrentTimeInt64(sqlite3_vfs*, sqlite3_int64*);
 
 static sqlite3_vfs jt_vfs = {
-  2,                             /* iVersion */
+  1,                             /* iVersion */
   sizeof(jt_file),               /* szOsFile */
   JT_MAX_PATHNAME,               /* mxPathname */
   0,                             /* pNext */
@@ -178,9 +181,7 @@ static sqlite3_vfs jt_vfs = {
   jtDlClose,                     /* xDlClose */
   jtRandomness,                  /* xRandomness */
   jtSleep,                       /* xSleep */
-  jtCurrentTime,                 /* xCurrentTime */
-  0,                             /* xGetLastError */
-  jtCurrentTimeInt64             /* xCurrentTimeInt64 */
+  jtCurrentTime                  /* xCurrentTime */
 };
 
 static sqlite3_io_methods jt_io_methods = {
@@ -205,28 +206,13 @@ struct JtGlobal {
 };
 static struct JtGlobal g = {0, 0};
 
-/*
-** Functions to obtain and relinquish a mutex to protect g.pList. The
-** STATIC_PRNG mutex is reused, purely for the sake of convenience.
-*/
-static void enterJtMutex(void){
-  sqlite3_mutex_enter(sqlite3_mutex_alloc(SQLITE_MUTEX_STATIC_PRNG));
-}
-static void leaveJtMutex(void){
-  sqlite3_mutex_leave(sqlite3_mutex_alloc(SQLITE_MUTEX_STATIC_PRNG));
-}
-
 extern int sqlite3_io_error_pending;
-extern int sqlite3_io_error_hit;
-static void stop_ioerr_simulation(int *piSave, int *piSave2){
+static void stop_ioerr_simulation(int *piSave){
   *piSave = sqlite3_io_error_pending;
-  *piSave2 = sqlite3_io_error_hit;
   sqlite3_io_error_pending = -1;
-  sqlite3_io_error_hit = 0;
 }
-static void start_ioerr_simulation(int iSave, int iSave2){
+static void start_ioerr_simulation(int iSave){
   sqlite3_io_error_pending = iSave;
-  sqlite3_io_error_hit = iSave2;
 }
 
 /*
@@ -250,12 +236,11 @@ static int jtClose(sqlite3_file *pFile){
   jt_file *p = (jt_file *)pFile;
 
   closeTransaction(p);
-  enterJtMutex();
   if( p->zName ){
     for(pp=&g.pList; *pp!=p; pp=&(*pp)->pNext);
     *pp = p->pNext;
   }
-  leaveJtMutex();
+
   return sqlite3OsClose(p->pReal);
 }
 
@@ -272,6 +257,7 @@ static int jtRead(
   return sqlite3OsRead(p->pReal, zBuf, iAmt, iOfst);
 }
 
+
 /*
 ** Parameter zJournal is the name of a journal file that is currently 
 ** open. This function locates and returns the handle opened on the
@@ -288,18 +274,16 @@ static int jtRead(
 **/
 static jt_file *locateDatabaseHandle(const char *zJournal){
   jt_file *pMain = 0;
-  enterJtMutex();
   for(pMain=g.pList; pMain; pMain=pMain->pNext){
-    int nName = (int)(strlen(zJournal) - strlen("-journal"));
+    int nName = strlen(zJournal) - strlen("-journal");
     if( (pMain->flags&SQLITE_OPEN_MAIN_DB)
-     && ((int)strlen(pMain->zName)==nName)
+     && (strlen(pMain->zName)==nName)
      && 0==memcmp(pMain->zName, zJournal, nName)
      && (pMain->eLock>=SQLITE_LOCK_RESERVED)
     ){
       break;
     }
   }
-  leaveJtMutex();
   return pMain;
 }
 
@@ -359,7 +343,6 @@ static int openTransaction(jt_file *pMain, jt_file *pJournal){
   sqlite3_file *p = pMain->pReal;
   int rc = SQLITE_OK;
 
-  closeTransaction(pMain);
   aData = sqlite3_malloc(pMain->nPagesize);
   pMain->pWritable = sqlite3BitvecCreate(pMain->nPage);
   pMain->aCksum = sqlite3_malloc(sizeof(u32) * (pMain->nPage + 1));
@@ -370,28 +353,18 @@ static int openTransaction(jt_file *pMain, jt_file *pJournal){
   }else if( pMain->nPage>0 ){
     u32 iTrunk;
     int iSave;
-    int iSave2;
 
-    stop_ioerr_simulation(&iSave, &iSave2);
+    stop_ioerr_simulation(&iSave);
 
     /* Read the database free-list. Add the page-number for each free-list
     ** leaf to the jt_file.pWritable bitvec.
     */
     rc = sqlite3OsRead(p, aData, pMain->nPagesize, 0);
-    if( rc==SQLITE_OK ){
-      u32 nDbsize = decodeUint32(&aData[28]);
-      if( nDbsize>0 && memcmp(&aData[24], &aData[92], 4)==0 ){
-        u32 iPg;
-        for(iPg=nDbsize+1; iPg<=pMain->nPage; iPg++){
-          sqlite3BitvecSet(pMain->pWritable, iPg);
-        }
-      }
-    }
     iTrunk = decodeUint32(&aData[32]);
     while( rc==SQLITE_OK && iTrunk>0 ){
       u32 nLeaf;
       u32 iLeaf;
-      sqlite3_int64 iOff = (i64)(iTrunk-1)*pMain->nPagesize;
+      sqlite3_int64 iOff = (iTrunk-1)*pMain->nPagesize;
       rc = sqlite3OsRead(p, aData, pMain->nPagesize, iOff);
       nLeaf = decodeUint32(&aData[4]);
       for(iLeaf=0; rc==SQLITE_OK && iLeaf<nLeaf; iLeaf++){
@@ -404,22 +377,96 @@ static int openTransaction(jt_file *pMain, jt_file *pJournal){
     /* Calculate and store a checksum for each page in the database file. */
     if( rc==SQLITE_OK ){
       int ii;
-      for(ii=0; rc==SQLITE_OK && ii<(int)pMain->nPage; ii++){
+      for(ii=0; rc==SQLITE_OK && ii<pMain->nPage; ii++){
         i64 iOff = (i64)(pMain->nPagesize) * (i64)ii;
         if( iOff==PENDING_BYTE ) continue;
         rc = sqlite3OsRead(pMain->pReal, aData, pMain->nPagesize, iOff);
         pMain->aCksum[ii] = genCksum(aData, pMain->nPagesize);
-        if( ii+1==pMain->nPage && rc==SQLITE_IOERR_SHORT_READ ) rc = SQLITE_OK;
       }
     }
 
-    start_ioerr_simulation(iSave, iSave2);
+    start_ioerr_simulation(iSave);
   }
 
   sqlite3_free(aData);
   return rc;
 }
 
+/*
+** Write data to an jt-file.
+*/
+static int jtWrite(
+  sqlite3_file *pFile, 
+  const void *zBuf, 
+  int iAmt, 
+  sqlite_int64 iOfst
+){
+  jt_file *p = (jt_file *)pFile;
+  if( p->flags&SQLITE_OPEN_MAIN_JOURNAL ){
+    if( iOfst==0 ){
+      jt_file *pMain = locateDatabaseHandle(p->zName);
+      assert( pMain );
+  
+      if( decodeJournalHdr(zBuf, 0, &pMain->nPage, 0, &pMain->nPagesize) ){
+        /* Zeroing the first journal-file header. This is the end of a
+        ** transaction. */
+        closeTransaction(pMain);
+      }else{
+        /* Writing the first journal header to a journal file. This happens
+        ** when a transaction is first started.  */
+        int rc;
+        if( SQLITE_OK!=(rc=openTransaction(pMain, p)) ){
+          return rc;
+        }
+      }
+    }
+    if( p->iMaxOff<(iOfst + iAmt) ){
+      p->iMaxOff = iOfst + iAmt;
+    }
+  }
+
+  if( p->flags&SQLITE_OPEN_MAIN_DB && p->pWritable ){
+    if( iAmt<p->nPagesize 
+     && p->nPagesize%iAmt==0 
+     && iOfst>=(PENDING_BYTE+512) 
+     && iOfst+iAmt<=PENDING_BYTE+p->nPagesize
+    ){
+      /* No-op. This special case is hit when the backup code is copying a
+      ** to a database with a larger page-size than the source database and
+      ** it needs to fill in the non-locking-region part of the original
+      ** pending-byte page.
+      */
+    }else{
+      u32 pgno = iOfst/p->nPagesize + 1;
+      assert( (iAmt==1||iAmt==p->nPagesize) && ((iOfst+iAmt)%p->nPagesize)==0 );
+      assert( pgno<=p->nPage || p->nSync>0 );
+      assert( pgno>p->nPage || sqlite3BitvecTest(p->pWritable, pgno) );
+    }
+  }
+
+  return sqlite3OsWrite(p->pReal, zBuf, iAmt, iOfst);
+}
+
+/*
+** Truncate an jt-file.
+*/
+static int jtTruncate(sqlite3_file *pFile, sqlite_int64 size){
+  jt_file *p = (jt_file *)pFile;
+  if( p->flags&SQLITE_OPEN_MAIN_JOURNAL && size==0 ){
+    /* Truncating a journal file. This is the end of a transaction. */
+    jt_file *pMain = locateDatabaseHandle(p->zName);
+    closeTransaction(pMain);
+  }
+  if( p->flags&SQLITE_OPEN_MAIN_DB && p->pWritable ){
+    u32 pgno;
+    u32 locking_page = (u32)(PENDING_BYTE/p->nPagesize+1);
+    for(pgno=size/p->nPagesize+1; pgno<=p->nPage; pgno++){
+      assert( pgno==locking_page || sqlite3BitvecTest(p->pWritable, pgno) );
+    }
+  }
+  return sqlite3OsTruncate(p->pReal, size);
+}
+
 /*
 ** The first argument to this function is a handle open on a journal file.
 ** This function reads the journal file and adds the page number for each
@@ -433,14 +480,13 @@ static int readJournalFile(jt_file *p, jt_file *pMain){
   sqlite3_int64 iSize = p->iMaxOff;
   unsigned char *aPage;
   int iSave;
-  int iSave2;
 
   aPage = sqlite3_malloc(pMain->nPagesize);
   if( !aPage ){
     return SQLITE_IOERR_NOMEM;
   }
 
-  stop_ioerr_simulation(&iSave, &iSave2);
+  stop_ioerr_simulation(&iSave);
 
   while( rc==SQLITE_OK && iOff<iSize ){
     u32 nRec, nPage, nSector, nPagesize;
@@ -466,7 +512,7 @@ static int readJournalFile(jt_file *p, jt_file *pMain){
           continue;
         }
       }
-      nRec = (u32)((iSize-iOff) / (pMain->nPagesize+8));
+      nRec = (iSize-iOff) / (pMain->nPagesize+8);
     }
 
     /* Read all the records that follow the journal-header just read. */
@@ -493,7 +539,7 @@ static int readJournalFile(jt_file *p, jt_file *pMain){
   }
 
 finish_rjf:
-  start_ioerr_simulation(iSave, iSave2);
+  start_ioerr_simulation(iSave);
   sqlite3_free(aPage);
   if( rc==SQLITE_IOERR_SHORT_READ ){
     rc = SQLITE_OK;
@@ -501,90 +547,6 @@ finish_rjf:
   return rc;
 }
 
-/*
-** Write data to an jt-file.
-*/
-static int jtWrite(
-  sqlite3_file *pFile, 
-  const void *zBuf, 
-  int iAmt, 
-  sqlite_int64 iOfst
-){
-  int rc;
-  jt_file *p = (jt_file *)pFile;
-  if( p->flags&SQLITE_OPEN_MAIN_JOURNAL ){
-    if( iOfst==0 ){
-      jt_file *pMain = locateDatabaseHandle(p->zName);
-      assert( pMain );
-  
-      if( iAmt==28 ){
-        /* Zeroing the first journal-file header. This is the end of a
-        ** transaction. */
-        closeTransaction(pMain);
-      }else if( iAmt!=12 ){
-        /* Writing the first journal header to a journal file. This happens
-        ** when a transaction is first started.  */
-        u8 *z = (u8 *)zBuf;
-        pMain->nPage = decodeUint32(&z[16]);
-        pMain->nPagesize = decodeUint32(&z[24]);
-        if( SQLITE_OK!=(rc=openTransaction(pMain, p)) ){
-          return rc;
-        }
-      }
-    }
-    if( p->iMaxOff<(iOfst + iAmt) ){
-      p->iMaxOff = iOfst + iAmt;
-    }
-  }
-
-  if( p->flags&SQLITE_OPEN_MAIN_DB && p->pWritable ){
-    if( iAmt<(int)p->nPagesize 
-     && p->nPagesize%iAmt==0 
-     && iOfst>=(PENDING_BYTE+512) 
-     && iOfst+iAmt<=PENDING_BYTE+p->nPagesize
-    ){
-      /* No-op. This special case is hit when the backup code is copying a
-      ** to a database with a larger page-size than the source database and
-      ** it needs to fill in the non-locking-region part of the original
-      ** pending-byte page.
-      */
-    }else{
-      u32 pgno = (u32)(iOfst/p->nPagesize + 1);
-      assert( (iAmt==1||iAmt==p->nPagesize) && ((iOfst+iAmt)%p->nPagesize)==0 );
-      assert( pgno<=p->nPage || p->nSync>0 );
-      assert( pgno>p->nPage || sqlite3BitvecTest(p->pWritable, pgno) );
-    }
-  }
-
-  rc = sqlite3OsWrite(p->pReal, zBuf, iAmt, iOfst);
-  if( (p->flags&SQLITE_OPEN_MAIN_JOURNAL) && iAmt==12 ){
-    jt_file *pMain = locateDatabaseHandle(p->zName);
-    int rc2 = readJournalFile(p, pMain);
-    if( rc==SQLITE_OK ) rc = rc2;
-  }
-  return rc;
-}
-
-/*
-** Truncate an jt-file.
-*/
-static int jtTruncate(sqlite3_file *pFile, sqlite_int64 size){
-  jt_file *p = (jt_file *)pFile;
-  if( p->flags&SQLITE_OPEN_MAIN_JOURNAL && size==0 ){
-    /* Truncating a journal file. This is the end of a transaction. */
-    jt_file *pMain = locateDatabaseHandle(p->zName);
-    closeTransaction(pMain);
-  }
-  if( p->flags&SQLITE_OPEN_MAIN_DB && p->pWritable ){
-    u32 pgno;
-    u32 locking_page = (u32)(PENDING_BYTE/p->nPagesize+1);
-    for(pgno=(u32)(size/p->nPagesize+1); pgno<=p->nPage; pgno++){
-      assert( pgno==locking_page || sqlite3BitvecTest(p->pWritable, pgno) );
-    }
-  }
-  return sqlite3OsTruncate(p->pReal, size);
-}
-
 /*
 ** Sync an jt-file.
 */
@@ -663,7 +625,7 @@ static int jtCheckReservedLock(sqlite3_file *pFile, int *pResOut){
 */
 static int jtFileControl(sqlite3_file *pFile, int op, void *pArg){
   jt_file *p = (jt_file *)pFile;
-  return p->pReal->pMethods->xFileControl(p->pReal, op, pArg);
+  return sqlite3OsFileControl(p->pReal, op, pArg);
 }
 
 /*
@@ -694,7 +656,6 @@ static int jtOpen(
 ){
   int rc;
   jt_file *p = (jt_file *)pFile;
-  pFile->pMethods = 0;
   p->pReal = (sqlite3_file *)&p[1];
   p->pReal->pMethods = 0;
   rc = sqlite3OsOpen(g.pVfs, zName, p->pReal, flags, pOutFlags);
@@ -707,12 +668,10 @@ static int jtOpen(
     p->pNext = 0;
     p->pWritable = 0;
     p->aCksum = 0;
-    enterJtMutex();
     if( zName ){
       p->pNext = g.pList;
       g.pList = p;
     }
-    leaveJtMutex();
   }
   return rc;
 }
@@ -723,7 +682,7 @@ static int jtOpen(
 ** returning.
 */
 static int jtDelete(sqlite3_vfs *pVfs, const char *zPath, int dirSync){
-  int nPath = (int)strlen(zPath);
+  int nPath = strlen(zPath);
   if( nPath>8 && 0==strcmp("-journal", &zPath[nPath-8]) ){
     /* Deleting a journal file. The end of a transaction. */
     jt_file *pMain = locateDatabaseHandle(zPath);
@@ -812,13 +771,7 @@ static int jtSleep(sqlite3_vfs *pVfs, int nMicro){
 ** Return the current time as a Julian Day number in *pTimeOut.
 */
 static int jtCurrentTime(sqlite3_vfs *pVfs, double *pTimeOut){
-  return g.pVfs->xCurrentTime(g.pVfs, pTimeOut);
-}
-/*
-** Return the current time as a Julian Day number in *pTimeOut.
-*/
-static int jtCurrentTimeInt64(sqlite3_vfs *pVfs, sqlite3_int64 *pTimeOut){
-  return g.pVfs->xCurrentTimeInt64(g.pVfs, pTimeOut);
+  return sqlite3OsCurrentTime(g.pVfs, pTimeOut);
 }
 
 /**************************************************************************
@@ -838,11 +791,6 @@ int jt_register(char *zWrap, int isDefault){
     return SQLITE_ERROR;
   }
   jt_vfs.szOsFile = sizeof(jt_file) + g.pVfs->szOsFile;
-  if( g.pVfs->iVersion==1 ){
-    jt_vfs.iVersion = 1;
-  }else if( g.pVfs->xCurrentTimeInt64==0 ){
-    jt_vfs.xCurrentTimeInt64 = 0;
-  }
   sqlite3_vfs_register(&jt_vfs, isDefault);
   return SQLITE_OK;
 }
@@ -850,7 +798,7 @@ int jt_register(char *zWrap, int isDefault){
 /*
 ** Uninstall the jt VFS, if it is installed.
 */
-void jt_unregister(void){
+void jt_unregister(){
   sqlite3_vfs_unregister(&jt_vfs);
 }
 
