@@ -10,6 +10,8 @@
 **
 *************************************************************************
 **
+** $Id: test_onefile.c,v 1.11 2009/02/10 18:54:03 danielk1977 Exp $
+**
 ** OVERVIEW:
 **
 **   This file contains some example code demonstrating how the SQLite 
@@ -26,8 +28,8 @@
 **   from sqlite3_malloc(). Any attempt to create a temporary database file 
 **   will fail (SQLITE_IOERR). To prevent SQLite from attempting this,
 **   it should be configured to store all temporary database files in 
-**   main memory (see pragma "temp_store" or the TEMP_STORE compile time
-**   option).
+**   main memory (see pragma "temp_store" or the SQLITE_TEMP_STORE compile 
+**   time option).
 **
 ** ASSUMPTIONS:
 **
@@ -79,7 +81,7 @@
 **   time using this demo vfs.
 */
 
-#include <sqlite3.h>
+#include "sqlite3.h"
 #include <assert.h>
 #include <string.h>
 
@@ -136,7 +138,7 @@ static int fsSync(sqlite3_file*, int flags);
 static int fsFileSize(sqlite3_file*, sqlite3_int64 *pSize);
 static int fsLock(sqlite3_file*, int);
 static int fsUnlock(sqlite3_file*, int);
-static int fsCheckReservedLock(sqlite3_file*);
+static int fsCheckReservedLock(sqlite3_file*, int *pResOut);
 static int fsFileControl(sqlite3_file*, int op, void *pArg);
 static int fsSectorSize(sqlite3_file*);
 static int fsDeviceCharacteristics(sqlite3_file*);
@@ -152,7 +154,7 @@ static int tmpSync(sqlite3_file*, int flags);
 static int tmpFileSize(sqlite3_file*, sqlite3_int64 *pSize);
 static int tmpLock(sqlite3_file*, int);
 static int tmpUnlock(sqlite3_file*, int);
-static int tmpCheckReservedLock(sqlite3_file*);
+static int tmpCheckReservedLock(sqlite3_file*, int *pResOut);
 static int tmpFileControl(sqlite3_file*, int op, void *pArg);
 static int tmpSectorSize(sqlite3_file*);
 static int tmpDeviceCharacteristics(sqlite3_file*);
@@ -162,12 +164,11 @@ static int tmpDeviceCharacteristics(sqlite3_file*);
 */
 static int fsOpen(sqlite3_vfs*, const char *, sqlite3_file*, int , int *);
 static int fsDelete(sqlite3_vfs*, const char *zName, int syncDir);
-static int fsAccess(sqlite3_vfs*, const char *zName, int flags);
-static int fsGetTempname(sqlite3_vfs*, int nOut, char *zOut);
+static int fsAccess(sqlite3_vfs*, const char *zName, int flags, int *);
 static int fsFullPathname(sqlite3_vfs*, const char *zName, int nOut,char *zOut);
 static void *fsDlOpen(sqlite3_vfs*, const char *zFilename);
 static void fsDlError(sqlite3_vfs*, int nByte, char *zErrMsg);
-static void *fsDlSym(sqlite3_vfs*,void*, const char *zSymbol);
+static void (*fsDlSym(sqlite3_vfs*,void*, const char *zSymbol))(void);
 static void fsDlClose(sqlite3_vfs*, void*);
 static int fsRandomness(sqlite3_vfs*, int nByte, char *zOut);
 static int fsSleep(sqlite3_vfs*, int microseconds);
@@ -192,7 +193,6 @@ static fs_vfs_t fs_vfs = {
     fsOpen,                                     /* xOpen */
     fsDelete,                                   /* xDelete */
     fsAccess,                                   /* xAccess */
-    fsGetTempname,                              /* xGetTempName */
     fsFullPathname,                             /* xFullPathname */
     fsDlOpen,                                   /* xDlOpen */
     fsDlError,                                  /* xDlError */
@@ -336,7 +336,8 @@ static int tmpUnlock(sqlite3_file *pFile, int eLock){
 /*
 ** Check if another file-handle holds a RESERVED lock on a tmp-file.
 */
-static int tmpCheckReservedLock(sqlite3_file *pFile){
+static int tmpCheckReservedLock(sqlite3_file *pFile, int *pResOut){
+  *pResOut = 0;
   return SQLITE_OK;
 }
 
@@ -543,8 +544,9 @@ static int fsUnlock(sqlite3_file *pFile, int eLock){
 /*
 ** Check if another file-handle holds a RESERVED lock on an fs-file.
 */
-static int fsCheckReservedLock(sqlite3_file *pFile){
-  return 0;
+static int fsCheckReservedLock(sqlite3_file *pFile, int *pResOut){
+  *pResOut = 0;
+  return SQLITE_OK;
 }
 
 /*
@@ -602,6 +604,7 @@ static int fsOpen(
   for(; pReal && strncmp(pReal->zName, zName, nName); pReal=pReal->pNext);
 
   if( !pReal ){
+    int real_flags = (flags&~(SQLITE_OPEN_MAIN_DB))|SQLITE_OPEN_TEMP_DB;
     sqlite3_int64 size;
     sqlite3_file *pRealFile;
     sqlite3_vfs *pParent = pFsVfs->pParent;
@@ -616,7 +619,7 @@ static int fsOpen(
     pReal->zName = zName;
     pReal->pFile = (sqlite3_file *)(&pReal[1]);
 
-    rc = pParent->xOpen(pParent, zName, pReal->pFile, flags, pOutFlags);
+    rc = pParent->xOpen(pParent, zName, pReal->pFile, real_flags, pOutFlags);
     if( rc!=SQLITE_OK ){
       goto open_out;
     }
@@ -698,7 +701,12 @@ static int fsDelete(sqlite3_vfs *pVfs, const char *zPath, int dirSync){
 ** Test for access permissions. Return true if the requested permission
 ** is available, or false otherwise.
 */
-static int fsAccess(sqlite3_vfs *pVfs, const char *zPath, int flags){
+static int fsAccess(
+  sqlite3_vfs *pVfs, 
+  const char *zPath, 
+  int flags, 
+  int *pResOut
+){
   fs_vfs_t *pFsVfs = (fs_vfs_t *)pVfs;
   fs_real_file *pReal;
   int isJournal = 0;
@@ -706,7 +714,7 @@ static int fsAccess(sqlite3_vfs *pVfs, const char *zPath, int flags){
 
   if( flags!=SQLITE_ACCESS_EXISTS ){
     sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
-    return pParent->xAccess(pParent, zPath, flags);
+    return pParent->xAccess(pParent, zPath, flags, pResOut);
   }
 
   assert(strlen("-journal")==8);
@@ -717,18 +725,9 @@ static int fsAccess(sqlite3_vfs *pVfs, const char *zPath, int flags){
 
   pReal = pFsVfs->pFileList; 
   for(; pReal && strncmp(pReal->zName, zPath, nName); pReal=pReal->pNext);
-  if( !pReal ) return 0;
-  return ((!isJournal||pReal->nJournal>0)?1:0);
-}
 
-/*
-** Populate buffer zBufOut with a pathname suitable for use as a 
-** temporary file. zBufOut is guaranteed to point to a buffer of 
-** at least (FS_MAX_PATHNAME+1) bytes.
-*/
-static int fsGetTempname(sqlite3_vfs *pVfs, int nBufOut, char *zBufOut){
-  sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
-  return pParent->xGetTempname(pParent, nBufOut, zBufOut);
+  *pResOut = (pReal && (!isJournal || pReal->nJournal>0));
+  return SQLITE_OK;
 }
 
 /*
@@ -767,9 +766,9 @@ static void fsDlError(sqlite3_vfs *pVfs, int nByte, char *zErrMsg){
 /*
 ** Return a pointer to the symbol zSymbol in the dynamic library pHandle.
 */
-static void *fsDlSym(sqlite3_vfs *pVfs, void *pHandle, const char *zSymbol){
+static void (*fsDlSym(sqlite3_vfs *pVfs, void *pH, const char *zSym))(void){
   sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
-  return pParent->xDlSym(pParent, pHandle, zSymbol);
+  return pParent->xDlSym(pParent, pH, zSym);
 }
 
 /*
@@ -822,4 +821,3 @@ int fs_register(){
 #ifdef SQLITE_TEST
   int SqlitetestOnefile_Init() {return fs_register();}
 #endif
-
